package PhotoPub::Config;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw($defaultalbum 
		$defaultalbumstyle $defaultrangestyle $defaultembedstyle
                $defaultpath %linkroots);

use strict;

use PhotoPub::Global qw(&register_init $error);
use PhotoPub::Utils qw($pathre $rangere
		       &checkpath &mkpath &remdups &untaint &md5_uri);
use PhotoPub::Handlers qw(&installhandler &forallhandlers 
			  &listhandlephoto &handlephoto);
use PhotoPub::Options qw(&initoptions &setoptions);
use PhotoPub::Ages qw(&updateage);
use PhotoPub::FileInfo qw(&mtime &getfiles &isdir);
use PhotoPub::Annots qw(&getannot);
use PhotoPub::Images qw(&canonicalname &picrange);
use PhotoPub::Cache qw(&exifinfo);

use vars qw($defaultalbum 
	    $defaultalbumstyle $defaultrangestyle $defaultembedstyle
            $defaultpath %linkroots);

use Config::General;
use HTML::Entities;
use Date::Parse;

#   **Warning**
# Our debug level must be less than or equal to that of our calling 
# script, or problems (eg output before headers etc) will occur
my $debug = 0;

my %defaultrefs=(
                 'defaultpath' => \$defaultpath,
                 'defaultalbum' => \$defaultalbum,
                 'defaultalbumstyle' => \$defaultalbumstyle,
		 'defaultrangestyle' => \$defaultrangestyle,
		 'defaultembedstyle' => \$defaultembedstyle
                );

my %templatetypes=('albumtemplates' => 'album',
		   'rangetemplates' => 'range',
		   'embedtemplates' => 'embed',
		  );

my %linkroottypes=(
                   'albumlinkroot' => 'album',
                   'rangelinkroot' => 'range',
                   'embedlinkroot' => 'embed',
                   'displaylinkroot' => 'display'
                  );

%linkroots=(
            'album' => 'album.pl',
            'range' => 'range.pl',
            'range' => 'embed.pl',
            'display' => 'display.pl'
           );

&register_init('config',sub {

		 my @scriptstat=stat($0) 
		   or &$error('config',"Couldn't stat $0");

		 my $config;

                 $config='/etc/photopubrc';

		 &installdefaulthandlers();
		 
		 &configprocess([],[],&readconfig($config));

                 if(defined $defaultpath) {
                    $defaultpath=$defaultpath.'/' 
                       if substr($defaultpath,-1,1) ne '/';
                 }
	       });

sub readconfig {
  my ($configfile)=@_;
  my %hash=();

  my $conf=eval { new Config::General(-file => $configfile,
                                      -DefaultConfig => \%hash); } 
    or &$error(type => 'config', 
	       message => "Error in config file $configfile : $@");
  return \%hash;
}

sub configprocess {
  my ($curpath,$basedir,$config)=@_;
  while (my ($item,$value)=each %$config) {
    if(exists $defaultrefs{$item}) {
      if(scalar @$basedir > 0 || scalar @$curpath > 0) {
#  make this a warning level error
	&$error(type => 'warning',
		message => "defaults can only be defined at the top level".
		           "of the root configuration file");
      }
      my $varref=$defaultrefs{$item};
      $$varref=&singlevalue($item,$value);
    } elsif(exists $linkroottypes{$item}) {
      if(scalar @$basedir > 0 || scalar @$curpath > 0) {
#  make this a warning level error
	&$error(type => 'warning',
		message => "link roots can only be defined at the top level".
		           "of the root configuration file");
      }
      $linkroots{$linkroottypes{$item}}=&singlevalue($item,$value);
    } elsif($item eq 'forcemogrify') {
      &setforcemogrify(&makepath($curpath),
		       &truthvalue(&singlevalue($item,$value)));
    } elsif($item eq 'exifwhentaken') {
      &exifwhentaken(&makepath($curpath));
    } elsif(exists $templatetypes{$item}) {
      $value=&verifyhash($item,$value);
      foreach my $style (keys %$value) {
	$value->{$style}=&dobasedir($basedir,$value->{$style});
	&installtemplates($templatetypes{$item},&makepath($curpath),
			  $value);
      }
    } elsif($item eq 'cachedir') {
      $value=&verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	$value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
	untaint(\($value->{'dir'}),$pathre,"directory must be a path");
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to cachedir");
      }
      &configurecachedir(&makepath($curpath),$value);
    } elsif($item eq 'exifcachedir') {
      &verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	$value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
	untaint(\($value->{'dir'}),$pathre,"directory must be a path");
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to exifcachedir");
      }
      &configureexifcachedir(&makepath($curpath),$value);
    } elsif($item eq 'sizecachedir') {
      &verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	$value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
	untaint(\($value->{'dir'}),$pathre,"directory must be a path");
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to sizecachedir");
      }
      &configuresizecachedir(&makepath($curpath),$value);
    } elsif($item eq 'imagedirs') {
      $value=&verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	$value->{'dir'} = [$value->{'dir'}] 
	  unless ref($value->{'dir'});
        for my $i (0..$#{$value->{'dir'}}) {
          $value->{'dir'}->[$i]=&dobasedir($basedir,$value->{'dir'}->[$i]);
	}
      } else {
	&$error(type => 'config',
		message => "Must specify at least one dir to imagedirs");
      }
      &configureimagedirs(&makepath($curpath),$value);
    } elsif($item eq 'aliasfile') {
      $value=verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	     $value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to aliasfile");
      }
      if(!exists $value->{'filename'}) {
	&$error(type => 'config',
		message => "Must specify filename option to aliasfile");
      }
      &configurealiasfile(&makepath($curpath),$value);
    } elsif($item eq 'includesfile') {
      &verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	     $value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to includesfile");
      }
      if(!exists $value->{'filename'}) {
	&$error(type => 'config',
		message => "Must specify filename option to includesfile");
      }
      &configureincludesfile(&makepath($curpath),$value);
    } elsif($item eq 'infofile') {
      $value=&verifyhash($item,$value);
      if(exists $value->{'dir'}) {
	     $value->{'dir'}=&dobasedir($basedir,$value->{'dir'});
      } else {
	&$error(type => 'config',
		message => "Must specify dir option to infofile");
      }
      if(!exists $value->{'filename'}) {
	&$error(type => 'config',
		message => "Must specify filename option to infofile");
      }
      &configureinfofile(&makepath($curpath),$value);
    } elsif($item eq 'album') {
      my $prefix=&makepath($curpath).'/';
      $value=&verifyhash($item,$value);
      while (my ($name,$subconfig)=each %$value) {
	$subconfig=&verifyhash("album $name",$subconfig);
	while (my ($option,$details)=each %$subconfig) {
	  if($option eq 'link') {
	    $details=&verifyhash("each link in $name",$details);
	    while (my ($album,$albumdetails)=each %$details) {
	      $albumdetails=&verifyhash("link $album in $name",$albumdetails);
	      if(!exists $albumdetails->{'title'}) {
		&$error(type => 'config',
			message => "Must specify title option in link to $album in $name");
	      }
	      my $level=exists $albumdetails->{'level'} ? 
		$albumdetails->{'level'} : 1;
	      &configurealbumlink($prefix.$name,
				  substr($album,0,1) eq '/' ? 
                                    $album : $prefix.$album,
				  $albumdetails->{'title'},
				  $level);
	    }
	  }
	  if($option eq 'include') {
	    my @includes;
	    if(ref($details) eq 'HASH') {
	      &$error(type => 'config',
		      message => "include option can't be a group in album $name");
	    } elsif(ref($details) eq 'ARRAY') {
	      @includes=@{$details};
	    } else {
	      @includes=($details);
	    }
	    foreach my $include (@includes) {
	      &configurealbuminclude($prefix.$name,
                                     substr($include,0,1) eq '/' 
                                        ? $include : $prefix.$include);
	    }
	  }
	}
      }
    } elsif($item eq 'path') {
      $value=&verifyhash($item,$value);
      while (my ($path,$subconfig)=each %$value) {
	$subconfig=&verifyhash("path $path",$subconfig);
	my @path=map { $_ ne '' ? ($_) : () } (split ('/',$path));
	&configprocess([@$curpath,@path],$basedir,$subconfig);
      }
    } elsif($item eq 'basedir') {
      $value=&verifyhash($item,$value);
      while (my ($dir,$subconfig)=each %$value) {
	$subconfig=&verifyhash("basedir $dir",$subconfig);
	my @dirs=map { $_ ne '' ? ($_) : () } (split ('/',$dir));
	&configprocess($curpath,[@$basedir,@dirs],$subconfig);
      }
    } elsif($item eq 'userconfig') {
      $value=&verifyhash($item,$value);
      my @users;
      if(exists $value->{'users'}) {
	@users=split(' ',$value->{'users'});
      } else {
	&$error(type => 'config',
		message => "Must specify users list for userconfig option");
      }
      if(exists $value->{'file'}) {
	my %loadedfiles=();
	my %allowedusers=();
	my %filenames=();
	foreach my $user (@users) {
	  $allowedusers{$user}=1;
	}
	&installhandler('dynamicloader',&makepath($curpath),sub {
			  my ($subpath)=@_;
			  my @pathelems=split('/',$subpath);
			  my $user=$pathelems[0];
                          return unless defined $user;
#                                         && defined $allowedusers{$user};
			  if(!defined $allowedusers{$user}) {
			    &$error(type => 'userinput',
				    message => "Unknown user $user");
			  }
			  my $file;
                          if(exists $filenames{$user}) {
                             $file=$filenames{$user};
                          } else {
                             $file=$value->{'file'};
			     my @pwdata=getpwnam($user)
			       or &$error(type => 'config',
				          message => "Unknown user $user");
			     my $home=$pwdata[7];
			     $file=~s/\$USER/$user/ieg;
			     $file=~s/\$HOME/$home/ieg;
                             $filenames{$user}=$file;
                          }
			  my $mtime=&mtime($file);
			  return if defined $loadedfiles{$user}
			              && $loadedfiles{$user}>=$mtime;
			  &configprocess([@$curpath,$user],$basedir,
					 &readconfig($file));
			  $loadedfiles{$user}=$mtime;
			});
      } else {
	&$error(type => 'config',
		message => "Must specify file for userconfig option");
      }
    } elsif($item eq 'include') {
      my $filename=&dobasedir($basedir,&singlevalue($item,$value));
      my $loadedtime=0;
      &installhandler('dynamicloader',&makepath($curpath),sub {
			my $mtime=&mtime($filename);
			return if $loadedtime>=$mtime;
			&configprocess($curpath,$basedir,
				       &readconfig($filename));
			$loadedtime=$mtime;
		      });
    } elsif($item eq 'templatealias') {
      my $target=&singlevalue($item,$value);
      $target="$target/" if substr($target,-1,1) ne '/';
      while(my ($plural,$type)=each %templatetypes) {
	my $handler=$type."template";
	# recursion breaking
	my $inside=0;
	my $path=&makepath($curpath);
	&installhandler($handler,$path,sub {
			  my ($subpath)=@_;
			  if($inside) {
			    &$error(type=>'config',
				    message=>'Possible recursion in '
                                              ."templatealias of $path/ to "
                                              ."$target");
			  }
			  $inside=1;
			  my $ret=&handlephoto($handler,$target.$subpath);
			  $inside=0;
			  return $ret;
			});
      }
    } else {
      # warn about unrecognised option
    }
  }
}

sub verifyhash {
  my ($item,$value)=@_;

  return {} unless defined $value;

  # in case it's empty
  if(!ref($value) && $value eq '') {
    return {};
  }
  if(ref($value) ne 'HASH') {
    &$error(type => 'config', 
	    message => "$item must be a grouping specifier");
  }
  return $value;
}

sub dobasedir {
  my ($basedir,$value)=@_;
  my $retdir = (substr($value,0,1) eq '/'
                ? $value 
	        : (&makepath($basedir).'/'.$value));
  chop $retdir if substr($retdir,-1,1) eq '/';
  return $retdir;
}

# Take Care: perl logic will often do the wrong thing if you aren't 
#            careful, as values of 0 mean things to us
sub truthvalue {
  my $value = shift;
  my $default = shift;

  # Don't do 'unless($default)' as $default=0 is meaningful to us
  unless(defined $default) { $default = 1; }
  if($default eq "")       { $default = 1; }

  unless(defined $value) { return $default; }
  if( $value eq '' )     { return $default; }

  my $firstchar=lc(substr($value,0,1));
  return 1 if $firstchar eq 'y' || $firstchar eq 't' || $firstchar eq '1';
  return 0 if $firstchar eq 'n' || $firstchar eq 'f' || $firstchar eq '0';
  &$error(type => 'config',
	  message => "Invalid value '$value' for boolean option");
}

sub makepath {
  my ($path)=@_;
  my $ret='';
  foreach my $entry (@$path) {
    $ret.="/$entry";
  }
  return $ret;
}

sub singlevalue {
  my ($item,$value)=@_;
  if(!ref($value)) {
    return $value;
  }
  elsif(ref($value) eq 'ARRAY') {
    &$error(type => 'config', 
	    message => "Multiple values specified for $item");
  } elsif(ref($value) eq 'HASH') {
    &$error(type => 'config',
	    message => "$item used as grouping specifier");
  } else {
    &$error(type => 'internal',
	    message => "reference value for $item which wasn't".
                       "an array or hash");
  } 
}


sub configurecachedir {
  my ($path,$options)=@_;
  my $dir=$options->{'dir'};
  my $digits=$options->{'digits'} || 0;
  my $prefix=$options->{'prefix'} || '';
  my $suffix=$options->{'suffix'} || '';

  installhandler('cachefileprefix',$path,sub {
		   my ($photo)=@_;
		   if(my ($subpath,$num)=($photo=~/^((?:$pathre\/)?)(\d+)$/)) {
		     mkpath($dir,$subpath);
		     return ("$dir/$subpath$prefix"
			     .'0' x ($digits-length($num))
			     .$num.$suffix);
		   } else {
		     &$error(type => 'userinput',
			     message => "Invalid photo $photo");
		   }
		 });
}

sub configureexifcachedir {
  my ($path,$options)=@_;
  my $dir=$options->{'dir'};
  my $digits=$options->{'digits'} || 0;
  my $prefix=$options->{'prefix'} || '';
  my $suffix=$options->{'suffix'} || '';

  &untaint(\$prefix,$pathre,"exifcache prefix must be a path");
  &untaint(\$digits,'\d+',"exifcache digits must be a number");
  &untaint(\$suffix,$pathre,"exifcache suffix must be a path");

  installhandler('exifcachefile',$path,sub {
		   my ($photo)=@_;
		   my ($subpath,$num)=($photo=~/((?:$pathre\/)?)(\d+)/);
		   &mkpath($dir,$subpath);
		   return "$dir/$subpath$prefix"."0" x ($digits-length($num))
		     .$num.$suffix;	    
		 });
}

sub configuresizecachedir {
  my ($path,$options)=@_;
  my $dir=$options->{'dir'};
  my $digits=$options->{'digits'} || 0;
  my $prefix=$options->{'prefix'} || '';
  my $suffix=$options->{'suffix'} || '';

  &untaint(\$prefix,$pathre,"sizecache prefix must be a path");
  &untaint(\$digits,'\d+',"sizecache digits must be a number");
  &untaint(\$suffix,$pathre,"sizecache suffix must be a path");

  installhandler('sizecachefile',$path,sub {
		   my ($photo)=@_;
		   my ($subpath,$num)=($photo=~/((?:$pathre\/)?)(\d+)/);
		   &mkpath($dir,$subpath);
		   return "$dir/$subpath$prefix"."0" x ($digits-length($num))
		     .$num.$suffix;	    
		 });
}

sub configureimagedirs {
  my ($path,$options)=@_;
  my $prefix = $options->{'prefix'} || '';
  my $digits = $options->{'digits'} || 0;
  my $suffix = $options->{'suffix'} || '';
  my $dirs = $options->{'dir'};

  if($debug >= 2) {
    print "imagefile: $path ".join(',',@$dirs)."\n";
  }

  installhandler('imagefile',$path,sub {
		   my ($photo)=@_;
		   my ($subpath,$num);
		   ($subpath,$num)=($photo=~/((?:.*\/)?)(\d+)/)
		     or &$error(type=>'userinput',
                                message=>qq{Invalid photo name "$path/$photo"});
		   my $file=$subpath.$prefix.'0' x ($digits-length($num))
		     .$num.$suffix;
		   foreach my $dir (@$dirs) {
		     return "$dir/$file" if &checkpath("$dir/$file");
		   }
		   return undef;
		 });
  installhandler('existsphotos',$path,sub {
		   my ($subpath)=@_;
		   return sub {
		     my ($nums)=@_;
		     my @ret=();
		     foreach my $num (@$nums) {
		       my $file=$subpath.'/'.$prefix.'0' x ($digits-length($num))
			 .$num.$suffix;
		       foreach my $dir (@$dirs) {
			 if(&checkpath("$dir/$file")) {
			   push @ret,$num;
			   last;
			 }
		       }
		     }
		     return \@ret;
		   }
		 });
  installhandler('photolist',$path,sub {
		   my ($subpath)=@_;
		   my $photos=[];
		   foreach my $dir (@$dirs) {
		     next if !checkpath("$dir/$subpath");

                     # Really ought to be elsewhere
                     unless($options->{'casesensitivet'}) {
                        $options->{'casesensitivet'} = 
                          truthvalue($options->{'casesensitive'});
                     }

		     foreach (&getfiles("$dir/$subpath")) {
                       if( $options->{'casesensitivet'} ) {
  		          if(/^$prefix(0*)([1-9]\d*)$suffix$/ 
			     && (length($1)+length($2)==$digits 
                                   || $digits==0)) {
  			    push @$photos,$2;
                          }
                       } else {
  		          if(/^$prefix(0*)([1-9]\d*)$suffix$/i 
			     && (length($1)+length($2)==$digits
                                   || $digits==0)) {
  			    push @$photos,$2;
                          }
		       }
		     }
		   }
		   return $photos;
		 });
  installhandler('enumsubpaths',$path,sub {
		   my ($subpath)=@_;
		   my %subpaths=();
		   foreach my $dir (@$dirs) {
		     next if !checkpath($dir);
		     foreach my $subdir (&enumsubdirs($dir,$subpath)) {
		       # switch / from beginning to end
		       $subdir=substr($subdir,1)."/" if $subdir; 

		       my $mtime=(stat($dir.'/'.$subdir))[9];
		       $subpaths{$subdir}=$mtime unless 
			 exists $subpaths{$subdir} 
			   && $subpaths{$subdir}>=$mtime;
		     }
		   }
		   return [sort { $subpaths{$a} <=> $subpaths{$b} } 
			   (keys %subpaths)];

		 });

}
		   
sub setforcemogrify {
  my ($path,$forcemogrify)=@_;
  &installhandler('forcemogrify',$path,sub {
		   return $forcemogrify;
		 });
}

sub exifwhentaken {
  my ($path)=@_;
  &installhandler('whentaken',$path,sub {
		    my ($subpath)=@_;
		    my $info=&exifinfo("$path/$subpath");
		    if(defined $info) {
		      my $whenstr=$info->{'DateTimeOriginal'};
		      if(defined $whenstr) {
			return &str2time($whenstr);
		      }
		    }
		    return undef;
		  });
}

sub installtemplates {
  my ($type,$path,$templates)=@_;
  &installhandler($type.'template',$path,sub {
		    my ($style)=@_;
		    return $templates->{$style};
		  });
}


sub configurealiasfile {
  my ($path,$options)=@_;

  my $aliasfilebasedir=$options->{'dir'};
  my $aliasfilename=$options->{'filename'};
  my $opaque
    =exists $options->{'opaque'} ? &truthvalue($options->{'opaque'}) : 0;

  my %aliases=();

  my %age=();

  my $parsealiasfile=sub {
    my ($subpath)=@_;
    my $aliasfile=$aliasfilebasedir.'/'.$subpath.$aliasfilename;

    return if defined $age{$subpath} 
           && -f $aliasfile 
           && &mtime($aliasfile)<=$age{$subpath};

    if(!open ALIAS,"<$aliasfile") {
      return;
    }
    $age{$subpath}=&mtime($aliasfile);

    $aliases{$subpath}={};

    my %defaultinherit=();

    while(<ALIAS>) {
      if(/^defaultinherit (.*)$/) {
	my @inherits=split(/\s+/,$1);
	%defaultinherit=();
	foreach my $annottype (@inherits) {
	  $defaultinherit{$annottype}=1;
	}
      } elsif(/^alias (\d+) ($pathre)(.*)$/) {
	my %inherit=();
        my $photo=$2;
        $photo="$path/$photo" unless substr($2,0,1) eq '/';
	if($3 eq '') {
	  %inherit=%defaultinherit;
	} else {
	  my @inherits=split(/\s+/,$3);
	  foreach my $annottype (@inherits) {
	    $inherit{$annottype}=1;
	  }    
	}
	$aliases{$subpath}->{$1}={'photo' => $photo,
				  'inherit' => \%inherit};
      } elsif(/^alias (\d+)\-(\d+) ((?:step \d+ )?)($pathre)(.*)$/) {
	my $sourcefrom=$1;
	my $sourceto=$2;
        my $sourcestepstr=$3;
        my $targetpath=$4;
        my $inherit=$5;
        my $sourcestep;

        ($sourcestep) = ($sourcestepstr =~ /^step (\d+) $/) or $sourcestep=1;

	my %inherit=();
	if($inherit eq '') {
	  %inherit=%defaultinherit;
	} else {
	  my @inherits=split(/\s+/,$inherit);
	  foreach my $annottype (@inherits) {
	    $inherit{$annottype}=1;
	  }    
	}

	my ($root,$targetfrom,$targetto) 
                   = $targetpath=~/^(?:(.*\/)?)(\d+)\-(\d+)$/ 
	  or &$error(type =>'internal',message=>"Couldn't parse $targetpath");

        $root="$path/$root" unless substr($2,0,1) eq '/';

	($targetto-$targetfrom)*$sourcestep == $sourceto-$sourcefrom
	  or &$error(type =>'config',
		     message=>
		     "Different sized source and target alias ranges in $_");
	
	my $targetnum=$targetfrom;
        my $sourcenum=$sourcefrom;
	while($targetnum<=$targetto) {
	  $aliases{$subpath}->{$sourcenum}={'photo' => $root.$targetnum,
					    'inherit' => \%inherit};
          $sourcenum+=$sourcestep;
          $targetnum++;
	}
      }
	
    }
    close ALIAS;
  };
  &setupaliases($path,\%aliases,$opaque,$parsealiasfile,$aliasfilebasedir,
		\%age);
}

sub configureincludesfile {
  my ($path,$options)=@_;

  my $includesfilebasedir=$options->{'dir'};
  my $includesfilename=$options->{'filename'};
  my $opaque
    =exists $options->{'opaque'} ? &truthvalue($options->{'opaque'}) : 0;

  my %aliases=();

  my %age=();

  my $parseincludesfile=sub {
    my ($subpath)=@_;
    my $includesfile=$includesfilebasedir.'/'.$subpath.$includesfilename;

    return if defined $age{$subpath} 
           && -f $includesfile 
           && &mtime($includesfile)<=$age{$subpath};

    if(!open INCLUDES,"<$includesfile") {
      return;
    }
    $age{$subpath}=&mtime($includesfile);

    $aliases{$subpath}={};

    my %defaultinherit=();
    my @includes=();

    while(<INCLUDES>) {
      if(/^defaultinherit (.*)$/) {
	my @inherits=split(/\s+/,$1);
	%defaultinherit=();
	foreach my $annottype (@inherits) {
	  $defaultinherit{$annottype}=1;
	}
      } elsif(/^include ($rangere)\s*((?:adjustwhentaken\s+[\+\-]?\d+)?)(.*)$/) {
	my %inherit=();
	my $range=$1;
	my $adjwhen=$2;
	my $inherits=$3;
	my $adjwhenfactor=0;
	if($adjwhen=~/^adjustwhentaken\s+\+?(\-?\d+)$/) {
	  $adjwhenfactor=$1;
	}
	if($inherits eq '') {
	  %inherit=%defaultinherit;
	} else {
	  my @inherits=split(/\s+/,$inherits);
	  foreach my $annottype (@inherits) {
	    $inherit{$annottype}=1;
	  }    
	}
	foreach my $photo (&picrange($range)) {
	  push @includes,{photo => $photo,
			  inherit => \%inherit,
			  adjustwhentaken => $adjwhenfactor};
	}
      } elsif(/^sortbywhentaken\s+$/) {
	foreach my $include (@includes) {
	  if(!exists $include->{'whentaken'}) {
	    $include->{'whentaken'}
	      =&handlephoto('whentaken',$include->{'photo'});
	    if(defined $include->{'whentaken'}) {
	      if(defined $include->{'adjustwhentaken'}) {
		$include->{'whentaken'}+=$include->{'adjustwhentaken'};
	      }
	    } else {
	      $include->{'whentaken'}=0;
	    }
	  }
	}
	@includes=sort { $a->{'whentaken'} <=> $b->{'whentaken'} } @includes;
      }
    }
    close INCLUDES;
    my $num=1;
    foreach my $include (@includes) {
      delete $include->{'whentaken'};
      $aliases{$subpath}->{$num}=$include;
      $num++;
    }
  };
  &setupaliases($path,\%aliases,$opaque,$parseincludesfile,
		$includesfilebasedir,\%age);
}


sub setupaliases {
  my ($path,$aliases,$opaque,$parsealiasfile,$aliasfilebasedir,$age)=@_;
  &installhandler('imagefile',$path,sub {
		    my ($photo)=@_;
		    my ($subpath,$num);
		    ($subpath,$num)=($photo=~/((?:.*\/)?)(\d+)/)
		     or &$error(type=>'userinput',
                                message=>qq{Invalid photo name "$path/$photo"});

		    &$parsealiasfile($subpath);

		    if(exists $aliases->{$subpath}->{$num}) {
		      &handlephoto('imagefile',
				   $aliases->{$subpath}->{$num}->{'photo'});
		    } else {
		      return undef;
		    }
		  });
  &installhandler('existsphotos',$path,sub {
		    my ($subpath)=@_;
		    &$parsealiasfile($subpath);
		    return sub {
		      my ($nums)=@_;
		      my @ret=();
		      # the stupid easy way to do it
		      foreach my $num (@$nums) {
			push @ret,$num 
			  if exists $aliases->{$subpath}->{$num}
			    && defined &handlephoto('imagefile',
				    $aliases->{$subpath}->{$num}->{'photo'});
		      }
		      return \@ret;
		    }
		  });
  &installhandler('photolist',$path,sub {
		    my ($subpath)=@_;

		    &$parsealiasfile($subpath);

		    my $photos=[];
		    if(exists $aliases->{$subpath}) {
		      foreach my $num (keys %{$aliases->{$subpath}}) {
			push @$photos,$num;
		      }
		    }
		    return $photos;
		  });

  &installhandler('enumsubpaths',$path,sub {
		    my ($subpath)=@_;
		    my %subpaths=();
		    my $dir=$aliasfilebasedir;
		    if(&checkpath($dir)) {
		      foreach my $subdir (enumsubdirs($dir,$subpath)) {
			my $mtime=(stat($dir.'/'.$subdir))[9];
			$subpaths{$subdir}=$mtime unless 
			  exists $subpaths{$subdir} 
			    && $subpaths{$subdir}>=$mtime;
		      }
		    }
		    return [sort { $subpaths{$a} <=> $subpaths{$b} } 
			    (keys %subpaths)];
		    
		  });

# lots of duplicated code patterns to clean up..

  &installhandler('cachefileprefix',$path,sub {
		    my ($photo)=@_;
		    my ($subpath,$num)=($photo=~/((?:$pathre\/)?)(\d+)/);
		    if(exists $aliases->{$subpath}->{$num}) {
		      return 
			&handlephoto('cachefileprefix',
				     $aliases->{$subpath}->{$num}->{'photo'}
				    );
		    } else {
		      return undef;
		    }
		  });

  if($debug >= 3) {
    print "canonical name for $path installed\n";
  }
  &installhandler('canonicalname',$path,sub {
		    my ($photo)=@_;
		    if($debug >= 3) {
		      print "canonical: $path/$photo\n";
		    }
		    my ($subpath,$num)=($photo=~/((?:$pathre\/)?)(\d+)/);
		    if(exists $aliases->{$subpath}->{$num}) {
		      my $next=$aliases->{$subpath}->{$num}->{'photo'};
		      return &canonicalname($next);
		    } else {
		      return undef;
		    }
		  }) unless $opaque;

  my $getannot=sub {
    my ($annottype,$photo)=@_;
    my ($subpath,$num);
    ($subpath,$num)=($photo=~/((?:.*\/)?)(\d+)$/)
      or &$error(type=>'internal',
                 message=>"Couldn't parse photo $photo");
    &$parsealiasfile($subpath);
    if(exists $aliases->{$subpath}->{$num}) { 
      &updateage($age->{$subpath});
      if(exists $aliases->{$subpath}->{$num}->{'inherit'}->{$annottype}) {
	return &getannot($aliases->{$subpath}->{$num}->{'photo'},$annottype);
      } 
    }
    return undef;
  };

  &installhandler('description',$path,sub {
		    return &$getannot('description',@_);
		  });
  
  &installhandler('options',$path,sub {
		    return &$getannot('options',@_);
		  });

  &installhandler('whentaken',$path,sub {
		    my ($photo)=@_;
		    my ($subpath,$num);
		    ($subpath,$num)=($photo=~/((?:.*\/)?)(\d+)$/)
		      or &$error(type=>'internal',
                                 message=>"Couldn't parse photo $photo");
		    &$parsealiasfile($subpath);
		    if(exists $aliases->{$subpath}->{$num}) {
		      my $aliasinfo=$aliases->{$subpath}->{$num};
		      my $whentaken=&handlephoto('whentaken',
						 $aliasinfo->{'photo'});
		      if(defined $whentaken) {
			if(exists $aliasinfo->{'adjustwhentaken'}) {
			  $whentaken+=$aliasinfo->{'adjustwhentaken'};
			}
			return $whentaken;
		      }
		    }
		    return undef;
		  });

		    

  &installhandler('test',$path,sub {
		    &$parsealiasfile(shift);
		  });

}



sub configureinfofile {
  my ($path,$options)=@_;

  my $infofilebasedir=$options->{'dir'};
  my $infofilename=$options->{'filename'};

# should now be correct
  my $allowhtml=&truthvalue($options->{'allowhtml'},0);
  my $allowdescl=&truthvalue($options->{'allowdescribelower'},0);
  my $listblankdir=&truthvalue($options->{'listblankdir'},0);
  my $dirdepthaslvl=&truthvalue($options->{'dirdepthaslevel'},0);

  my %annotations=();
  my %groups=();
  my %titles=();

  my %age=();

  my $setannotdescr=sub {
    my ($subpath,$num,$descr)=@_;
    $annotations{$subpath}->{$num}={} 
      unless defined $annotations{$subpath}->{$num};
    $annotations{$subpath}->{$num}->{'description'}=
      $allowhtml ? $descr : &encode_entities($descr);
  };
  
  my $setannotoptions=sub {
    my ($subpath,$num,$options)=@_;
    $annotations{$subpath}->{$num}={} 
      unless defined $annotations{$subpath}->{$num};
    $annotations{$subpath}->{$num}->{'options'}=&initoptions() 
      unless defined $annotations{$subpath}->{$num}->{'options'};
    setoptions($annotations{$subpath}->{$num}->{'options'},$options);
  };

  my $parseinfofile;
  $parseinfofile=sub {
    my ($subpath)=@_;
    my $infofile=$infofilebasedir.'/'.$subpath.$infofilename;

    # Should we load our parent's file?
    if( $allowdescl && $subpath ne "/" && $subpath ) {
        my($parent,$mpath) = ($subpath =~ /(.*\/)([\w\-\_\.]+\/)/);
        if($parent eq "/") { $parent = ""; }
        # Use %groups{$subpath} as a test for already having loaded
        unless($groups{$parent}) {
           &$parseinfofile($parent);
        }
    } 

    # Check if file has changed
    return if defined $age{$subpath} 
           && -f $infofile 
           && &mtime($infofile)<=$age{$subpath};

    $groups{$subpath}=[]
      unless defined $groups{$subpath};
    $annotations{$subpath}={}
      unless defined $annotations{$subpath};
    $titles{$subpath}={}
      unless defined $titles{$subpath};

    my $fullpath="$path/$subpath";
    chop $fullpath;
    
    my $photos = &listhandlephoto('photolist',$fullpath);
    my $photocount = scalar @$photos;

    if(!open INFO,"<$infofile") {
      # no info file for this directory found, make up a basic default group
      # which contains all the photos in the relevant subdirectory, as long
      # as there is at least one, or the user wants blank directories

      my $lvl = 1;
      if( $dirdepthaslvl ) {
         my @parts=split(/\//,$fullpath);
         $lvl = scalar(@parts) - 1;
      }

      if( $photocount>=1 || $listblankdir ) {
         push( @{$groups{$subpath}},{'range' => $subpath.'*',
				 'description' => "Directory $fullpath",
				 'level' => $lvl} );
      }
      $age{$subpath}=0;
      return;
    }

    if( $photocount == 0 ) {
      # Check for subdirs
      my $subcount = 0;

      forallhandlers('enumsubpaths',$fullpath,sub {
		    my ($thispath,$subpaths)=@_;
		    foreach my $thissubpath (@$subpaths) {
			$subcount++;
		    }
	     });

      &$error(type => 'warning',
           message => "No pictures found in path $fullpath") if $subcount==0;
    }

    $age{$subpath}=&mtime($infofile);
    my $pre = "";
    if( $allowdescl ) { $pre = '[\w\-\_\.\/]+\/'; }
    while(<INFO>) {
      if(/^group \/?($pre)?([\d\-\,\*]+) (.*)$/ ) {
        my ($workpath,$rng,$desc) = ($subpath.$1,$2,$3);

        unless($workpath) {$workpath="";}
        my $range=$workpath.$rng;

	push(@{$groups{$workpath}},{'range' => $range, 
				   'description' => 
				   $allowhtml ? $desc 
                                       : &encode_entities($desc),
				   'level' => 1});
      }
      if(/^subgroup (\d+) \/?($pre)?([\d\-\,\*]+) (.*)$/) {
        my ($lvl,$workpath,$rng,$desc) = ($1,$subpath.$2,$3,$4);

        my $range=$workpath.$rng;
	push(@{$groups{$workpath}},{'range' => $range,
				   'description' => $allowhtml ? $desc 
                                                     : &encode_entities($desc),
				   'level' => $lvl});
      }
      if(/^describe \/?($pre)?(\d+) (.*)$/) {
        my ($workpath,$num,$desc) = ($subpath.$1,$2,$3);
	&$setannotdescr($workpath,$num,$desc);
      }
      if(/^adjust \/?($pre)?(\d+) ([\d\.]+) ([\d\.]+) ([\d\.]+)/)
	{
          my ($workpath,$num,$red,$green,$blue) = ($subpath.$1,$2,$3,$4,$5);
	  &$setannotoptions($workpath,$num,{'red' => $red, 
					 'green' => $green, 
					 'blue' => $blue});
	}
      if(/^rotate \/?($pre)?(\d+)\s+([\d\.]+)\s*/) {
        my ($workpath,$num,$rot) = ($subpath.$1,$2,$3);
	&$setannotoptions($workpath,$num,{'rotate' => $rot});
      }
      if(/^rotate \/?($pre)?(\d+)\s+([\d\.]+)\s+autocrop\s*/) {
        my ($workpath,$num,$rot) = ($subpath.$1,$2,$3);
	&$setannotoptions($workpath,$num,{'rotate' => $rot.'a'});
      }
      if(/^gamma \/?($pre)?(\d+) ([\d\.]+)/) {
        my ($workpath,$num,$gamma) = ($subpath.$1,$2,$3);
	&$setannotoptions($workpath,$num,{'gamma' => $gamma});
      }
      if(/^crop \/?($pre)?(\d+) ([x\d\+\-\%]+)/) {
        my ($workpath,$num,$crop) = ($subpath.$1,$2,$3);
	&$setannotoptions($workpath,$num,{'crop' => $crop});
      }
    }
    close INFO;
  };

  installhandler('photogroups',$path,sub {
		   my ($subpath)=@_;
		   my $allgroups=[];
		   my @subpaths=();
		   my $fullpath=$path.'/'.$subpath;
		   return if !checkpath($fullpath);
		   chop $fullpath;
		   forallhandlers('enumsubpaths',$fullpath,sub {
				    my ($thispath,$subpaths)=@_;
				    foreach my $thissubpath (@$subpaths) {
				      push @subpaths,
					substr($thispath.$thissubpath,
					       length($path));
				    }
				  });

		   foreach my $thissubpath (@subpaths) {
		     &$parseinfofile($thissubpath);
		     updateage($age{$thissubpath});
		     push @$allgroups,@{$groups{$thissubpath}};
		   }
		   return $allgroups;
		 });

  my $getannot=sub {
    my ($annottype,$photo)=@_;
    my ($subpath,$num);
    ($subpath,$num)=($photo=~/((?:.*\/)?)(\d+)$/)
      or &$error(type=>'internal',
                 message=>"Couldn't parse photo $photo");
    &$parseinfofile($subpath);
    if(exists $annotations{$subpath}->{$num}) { 
      updateage($age{$subpath});
      return $annotations{$subpath}->{$num}->{$annottype};
    } else {
      return undef;
    }
  };

  installhandler('description',$path,sub {
		   return &$getannot('description',@_);
		 });

  installhandler('options',$path,sub {
		   return &$getannot('options',@_);
		 });

  installhandler('test',$path,sub {
		   &$parseinfofile(shift);
		 });

  if(exists $options->{'makealbum'}) {
    my $makealbum=$options->{'makealbum'};
    my @albums=ref($makealbum) eq "ARRAY" ? @{$makealbum} : ($makealbum);
    foreach my $album (@albums) {
      installhandler('album',"$path/$album",sub {
		       # ignore the residual path
		       my $allitems=[];
		       my @subpaths=();
		       forallhandlers('enumsubpaths',$path,sub {
					my ($thispath,$subpaths)=@_;
					foreach my $thissubpath (@$subpaths) {
					  push @subpaths,
					    substr($thispath.$thissubpath,
						   length($path));
					}
				      });

		       # Ensure we get the paths in a sensible order
		       @subpaths = sort(@subpaths);
		       
		       foreach my $thissubpath (@subpaths) {
			 &$parseinfofile($thissubpath);
			 updateage($age{$thissubpath});
			 foreach my $group (@{$groups{$thissubpath}}) {
			   my $item={type => 'group',
				     range => $path.'/'.$group->{'range'},
				     description => $group->{'description'},
				     level => $group->{'level'}
				    };
			   push @$allitems,$item;
			 }
		       }
		       return $allitems;
		     });
      installhandler('rangetitle',"$path/$album",sub {
		       # ignore the residual path
		       return sub {
			 my ($range)=@_;
			 my @subpaths=();

			 forallhandlers('enumsubpaths',$path,sub {
					  my ($thispath,$subpaths)=@_;
					  foreach my $thissubpath
					    (@$subpaths) {
					      push @subpaths,
						substr($thispath.$thissubpath,
						       length($path));
					    }
					});

			 # perhaps the list of groups should be precomputed 
			 # instead of doing a linear search each time?

			 foreach my $thissubpath (@subpaths) {
			   &$parseinfofile($thissubpath);

			   foreach my $group (@{$groups{$thissubpath}}) {
			     if($range eq $path.'/'.$group->{'range'}) {
			       &updateage($age{$thissubpath});
			       return $group->{'description'};
			     }
			   }
			 }
			 return undef;
		       }
		     });
    }
  }
}

sub configurealbumlink {
  my ($from,$to,$title,$level)=@_;
  &installhandler('album',$from,sub {
		    return [{type => 'link',
			     album => $to,
			     title => $title,
			     level => $level}];
		  });
}

sub configurealbuminclude {
  my ($from,$include)=@_;
  &installhandler('album',$from,sub {
		    return &listhandlephoto('album',$include);
		  });
  &installhandler('rangetitle',$from,sub {
		    # deliberately not using parameterhandlephoto here because
		    # we can just return the same subroutine; no need to go to
		    # all the hassle of essentially eta-expanding it.
		    return &handlephoto('rangetitle',$include);
		  });
}

sub enumsubdirs {
  my ($dir,$subpath)=@_;
  my @ret;
  if(&isdir("$dir$subpath")) {
    return if !&checkpath("$dir$subpath");
    push @ret,$subpath;
    my @subdirs=&getfiles("$dir$subpath");
    foreach my $subdir (@subdirs) {
      next if $subdir eq '.' || $subdir eq '..';
      push @ret,&enumsubdirs($dir,"$subpath/$subdir");
    }
  }
  return @ret;
}

sub installdefaulthandlers {
  &installhandler('cachefileprefix','',sub {
		    my ($photo)=@_;
		    &$error(type => 'config',
			    message => 'No cache handler installed for /'.
			               $photo);
		  });
  
  &installhandler('forcemogrify','',sub {
		    return 0;
		  });
}


1;
