package PhotoPub::DisplayHelper;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(&checkimageoptions);

use PhotoPub::Utils qw(&untaint);

my %image_entries=
  (
   'shrink' => ['\d+','shrink must be a positive integer'],
   'gamma' => ['[\d\.]+','gamma must be a positive number'],   
   'red' => ['[\d\.]+','red must be a positive number'],
   'green' => ['[\d\.]+','green must be a positive number'],
   'blue' => ['[\d\.]+','blue must be a positive number'],
   'rotate' => ['[\d\.]+a?',"rotate must be a positive integer "
		."with an optional 'a' at the end to select automatic"
		."cropping"],
   'thumb' => ['\d+x\d+','thumb must in the format <m>x<n> where '
	       .'<m> and <n> are positive integers'],
   'crop' => ['[x\d\+\-\%]+','crop must match the regexp [x\d\+\-\%]+'],
   'sharpen' => ['[\d\.]+','sharpen must be a positive number'],
   'quality' => ['\d+','quality must be a positive integer']
  );

sub checkimageoptions {
  my ($query)=@_;
  my %output=();

  foreach my $entry (keys %image_entries) {
    my $value=$query->param($entry);
    if(defined $value) {
      untaint(\$value,@{$image_entries{$entry}});
      $output{$entry}=$value;
    }
  }

  &usererror("shrink must be between 1 and 100") 
    if(defined $output->{'shrink'} && 
       ($output->{'shrink'}<1 || $output->{'shrink'}>100));

  if(defined $image_options->{'rotate'}) {
    $output->{'rotate'}=~/^(\d+)/;
    &usererror("rotate must be 0 or greater and less than 360")
      if $1<0 || $1>359;
  }

  if(defined $output->{'sharpen'}) {
    $output->{'sharpen'}=~/^(\d+)/;
    &usererror("sharpen must be 0 or greater and less than 100")
      if $1<0 || $1>99;
  }
  return \%output;
}
