package PhotoPub::RangeHelper;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(&dispahref &dispembedahref &dispimg &dispinfo);

use PhotoPub::Config qw(%linkroots);
use PhotoPub::Cache qw(&imagesize);
use PhotoPub::Images qw(&canonicalname);
use PhotoPub::Utils qw(&untaint &uri_escape &handletagoptions);


use strict;

sub dispurl {
  my ($photo,$options,$extra,$styleroot)=@_;
  my $url=$linkroots{'display'};
  $url.="?photo=".&uri_escape(&canonicalname($photo));
  foreach my $entry (keys %$options) {
    $url.='&amp;'.&uri_escape($entry)."=".&uri_escape($options->{$entry});
  }
  &handletagoptions(\$url,$styleroot,$extra);
  return $url;
}

sub dispembedurl {
  my ($photo,$options,$extra,$styleroot)=@_;
  my $url=$linkroots{'embed'};
  $url.="?photo=".&uri_escape(&canonicalname($photo));
  foreach my $entry (keys %$options) {
    $url.='&amp;'.&uri_escape($entry)."=".&uri_escape($options->{$entry});
  }
  &handletagoptions(\$url,$styleroot,$extra);
  return $url;
}

sub dispahref {
  my ($other,$photo,$options,$extra,$styleroot)=@_;
  return '<a'.$other.'href="'.dispurl($photo,$options,$extra,$styleroot).'">';
}

sub dispembedahref {
  my ($other,$photo,$options,$extra,$styleroot)=@_;
  return '<a'.$other.'href="'.dispembedurl($photo,$options,$extra,$styleroot)
             .'">';
}

sub dispimg {
  my ($other,$photo,$options,$extra,$styleroot)=@_;
  my %imgoptions=defined $options ? %{$options} : ();

  my $tagend;
  if (substr($extra,-1) eq '/') {
     $tagend=' />';
     chop $extra;
  } else {
     $tagend='>';
  }  

  foreach my $attrpair (split (/\s+/,$extra)) {
    if(my ($attr,$value) = split (/=/,$attrpair)) {
      $imgoptions{$attr}=$value;
    }
  }

  my ($width,$height)=&imagesize($photo,\%imgoptions);
  return '<img'.$other.'src="'.&dispurl($photo,$options,$extra,$styleroot).
                '" alt="Photo '.&canonicalname($photo).'" width="'.$width.
		  '" height="'.$height.'"'.$tagend;
}

sub dispinfo {
  my $photo = shift;
  my $desc = shift || "";
  my $out = '<p><input type="text" name="desc'
              .&uri_escape($photo).'" size=40 value="'.$desc.'">';
  $out .= '<select name="effect'.&uri_escape($photo).'">';
  foreach (qw(none rotate90 rotate180 rotate270)) {
    $out .= " <option value=\"$_\"";
    $out .= " selected" if $_ eq "none";
    $out .= ">$_</option>";
  }
  $out .= "</select>";
  return $out;
}
