#!/usr/bin/perl -wT

use lib '/usr/lib/photopub';

use strict;

use PhotoPub::Global qw(&init $error);
use PhotoPub::CGIBase qw(&usererror $query &printheader);
use PhotoPub::Config qw($defaultembedstyle %linkroots $defaultpath);
use PhotoPub::Handlers qw(&handlephoto &parameterhandlephoto &listhandlephoto);
use PhotoPub::Utils qw(&slurp &doifcount &untaint &uri_escape 
		       &handleparameters &includetag &excludetag
                       &parserange);
use PhotoPub::Images qw(&picfile &picrange &canonicalname);
use PhotoPub::Cache qw(&imagesize);
use PhotoPub::Annots qw(&getannot);
use PhotoPub::Ages qw(&updateagefromfiles);
use PhotoPub::RangeHelper qw(&dispahref &dispembedahref &dispimg &dispinfo);
use PhotoPub::DisplayHelper qw(&checkimageoptions);

use HTML::Entities;

my $debug = 0;

&init();

if( $debug >= 1 ) {
   # Do the headers early, so we can output to the browser soon
   # Might lead to some of the headers being wrong though
   &printheader();
   print "<pre>\n";
}

my $photo=$query->param('photo') or &usererror('Must give a photo!');

$photo=$defaultpath.'/'.$photo if substr($photo,0,1) ne '/';

my $style=$query->param('style') || $defaultembedstyle;

my $embedtemplate=&handlephoto('embedtemplate',$style);

defined $embedtemplate or &usererror('Style not defined!');

my $template=slurp($embedtemplate) 
  or &$error(type => 'config', message => "Couldn't open template file");

my $options=&checkimageoptions($query);

my @params=&handleparameters($query,\$template);

if($template=~s/\<userange lowest\>//i) {
  my $album=$query->param('album');
  my $items=&listhandlephoto('album',$album);
  my $wantedphoto=&canonicalname($photo);
  my $bestlevel=-1;
  my $besttitle="";
  my $bestrange="";
  foreach my $item (@$items) {
    my $choosethis = sub {
      $bestlevel=$item->{'level'};
      $besttitle=$item->{'description'};
      $bestrange=$item->{'range'};
    };
    if($item->{'type'} eq 'group' && $item->{'level'} > $bestlevel) {
      &parserange($item->{'range'},$defaultpath,
		  sub {
		    my ($foundpath,$foundnum)=@_;
		    my $foundphoto=$foundpath.$foundnum;
		    if(&canonicalname($foundphoto) eq $wantedphoto) {
		      &$choosethis();
		      return 0;
		    }
		    return 1;
		  });
    }
  }
  if($bestlevel==-1) {
    &$error(type=>'userinput',message=>'photo is not listed in a range');
  }
  $template=~s/\$rangetitle/$besttitle/gi;
  $template=~s/\$range/$bestrange/gi;
}

# Export $styleroot, the path of the stylesheet, into the template
# This should go away if we can find a nice way to do relative style 
# references here like in album.pl
my $styleroot = substr( $style, 0, rindex( $style, '/' ) );
$template=~s/\$styleroot/$styleroot/ieg;

&updateagefromfiles($0,$embedtemplate);

$template=~s/\$photo/&encode_entities(&canonicalname($photo))/ieg;

my $descr=getannot($photo,'description');
if(defined $descr) {
  &includetag('ifdescr',\$template);
  &excludetag('ifnodescr',\$template);
  # allowhtml setting will already have caused $descr to be HTML escaped 
  # if necessary
  $template=~s/\$description/$descr/ig;
} else {
  &excludetag('ifdescr',\$template);
  &includetag('ifnodescr',\$template);
  $template=~s/\$description//ig;
}

$template=~s/\<img(.*?)picture(.*?)\>/&dispimg($1,$photo,$options,$2,$styleroot)/gies;
$template=~s/\<a(.*?)picture(.*?)\>/&dispahref($1,$photo,$options,$2,$styleroot)/gies;
$template=~s/\<editinfo\>/&dispinfo($photo,$descr)/gies;

if( $debug == 0 ) {
  # We now know everything for the headers
  &printheader();
}

print $template;

