#!/usr/bin/perl -wT

use lib '/usr/lib/photopub';

use strict;

use PhotoPub::Global qw(&init $error);
use PhotoPub::CGIBase qw(&usererror $query &printheader);
use PhotoPub::Config qw($defaultrangestyle %linkroots);
use PhotoPub::Handlers qw(&handlephoto &parameterhandlephoto);
use PhotoPub::Utils qw(&slurp &doifcount &untaint &uri_escape 
		       &handleparameters &includetag &excludetag
                       &handlepages &handlesearch);
use PhotoPub::Images qw(&picfile &picrange &canonicalname);
use PhotoPub::Cache qw(&imagesize);
use PhotoPub::Annots qw(&getannot);
use PhotoPub::Ages qw(&updateagefromfiles);
use PhotoPub::RangeHelper qw(&dispahref &dispembedahref &dispimg &dispinfo);

use HTML::Entities;

my $debug = 0;

&init();

if( $debug >= 1 ) {
   # Do the headers early, so we can output to the browser soon
   # Might lead to some of the headers being wrong though
   &printheader();
   print "<pre>\n";
}

my $range=$query->param('range') or &usererror('Must supply range parameter');

my $style=$query->param('style') || $defaultrangestyle;

my $rangetemplate=&handlephoto('rangetemplate',$style);

my $search=$query->param('search');
if(defined $search) {
   $search = &handlesearch($search);
}

defined $rangetemplate or &usererror('Style not defined!');

my $template=slurp($rangetemplate) 
  or &$error(type => 'config', message => "Couldn't open template file");

my @params=&handleparameters($query,\$template);

# Export $styleroot, the path of the stylesheet, into the template
# This should go away if we can find a nice way to do relative style 
# references here like in album.pl
my $styleroot = substr( $style, 0, rindex( $style, '/' ) );
$template=~s/\$styleroot/$styleroot/ieg;

&updateagefromfiles($0,$rangetemplate);

my @photos=&picrange($range);

# Do search now
if(defined $search) {
   my @allphotos = @photos;
   @photos = ();
   foreach my $pic (@allphotos) {
      my $descr=getannot($pic,'description');
      if($descr =~ /$search/i) {
         push @photos,$pic;
      }
   }
}

# Split the range into multiple pages if requested to do so
my $startnum=0;
my $endnum=scalar @photos - 1;

if( $debug >= 1 ) {
  print "<pre>";
  foreach my $ph (@photos) {
    print $ph."\n";
  }
  print "$endnum\n</pre>";
}

#    Do proper blank range page here
#    -------------------------------
if($endnum == -1 ) {
  &usererror("That range \($range\) contains no entries - perhaps ". 
   "you specified an incorrect range?");
}


# string for links to different pages of the same range, if needed

my $rangequery;

&handlepages($query,\$template,\$startnum,\$endnum,
	     sub {
	       $rangequery=$linkroots{'range'}.'?range='.&uri_escape($range);
	       if(defined $query->param('album')) {
		 $rangequery.="&amp;album="
		   .&uri_escape($query->param('album'));
	       }
	       $rangequery.="&amp;style=".&uri_escape($style);
	       foreach my $param (@params) {
		 $rangequery.='&amp;'.&uri_escape($param)
		   .'='.&uri_escape($query->param($param));
	       }
	     },
	     sub {
	       my ($pagenum)=@_;
	       return "$rangequery&amp;page=$pagenum";
	     });
  
# shorten @photos to account for the pages
@photos = @photos[$startnum..$endnum];

if( $debug >= 1 ) {
  print "<pre>\nstart $startnum\nend $endnum\n";
  foreach my $p (@photos) {
    print $p."\n";
  }
  print "</pre>\n";
}

my $title;
if(defined $query->param('album')) {
  $title=&parameterhandlephoto('rangetitle',
			       $query->param('album'),
			       $range);
}

# perhaps HTML should be stripped completely?
if(defined $title) {
  &includetag('iftitle',\$template);
  &excludetag('ifnotitle',\$template);
  $template=~s/\$title/&encode_entities($title)/ieg;
} else {
  &excludetag('iftitle',\$template);
  &includetag('ifnotitle',\$template);
  $template=~s/\$title//ig;
}

$template=~s/\$range/&encode_entities($range)/ieg;

my $out="";
my ($pre,$repeat,$options,$post);

while(($pre,$options,$repeat,$post)
      =($template=~/^(.*?)\<REPEAT PICTURE(.*?)\>(.*?)\<\/REPEAT\>(.*)$/si)) {

  my $count=1;

  my $dophoto = sub {
    my ($photo,$section)=@_;
    my $descr=getannot($photo,'description');
    my $options=getannot($photo,'options');
    if(defined $descr) {
      &includetag('ifdescr',\$section);
      # allowhtml setting will already have caused $descr to be HTML escaped 
      # if necessary
      $section=~s/\$description/$descr/ig;
    } else {
      &excludetag('ifdescr',\$section);
      $section=~s/\$description//ig;
    }
    # We control count and it's only numbers => not escaped
    $section=~s/\$counter/$count/gi;
    my $filesizek=int((-s picfile($photo)) / 1024);
    # likewise filesizek is a number.
    $section=~s/\$filesizek/$filesizek/gi;
    $section=~s/\$picid/&encode_entities(&canonicalname($photo))/gie;
    $section=~s/\<img(.*?)picture(.*?)\>/&dispimg($1,$photo,$options,$2,$styleroot)/gies;
    $section=~s/\<a(.*?)picture(.*?)\>/&dispahref($1,$photo,$options,$2,$styleroot)/gies;
    $section=~s/\<a(.*?)embed(.*?)\>/&dispembedahref($1,$photo,$options,$2,$styleroot)/gies;
    $section=~s/\<editinfo\>/&dispinfo($photo,$descr)/gies;
    $section=&doifcount($section,$count,$#photos+1);
    $out.=$section;
    $count++;
  };

  $out.=$pre;

  my $innercount=1;
  if($options=~/inner\=(\d+)/si) { $innercount=$1; }

  if($innercount==1) {
    foreach my $photo (@photos) {
      &$dophoto($photo,$repeat);
    }
  } else {
    my @tempphotos=@photos;

    while (@tempphotos) {
      my @thesephotos=();
      my $i=0;
      while($i<$innercount && @tempphotos) {
	push @thesephotos,shift @tempphotos;
	$i++;
      }

      my $temprepeat=$repeat;
      my ($preinner,$inner,$postinner);

      while( ($preinner,$inner,$postinner)
	     =($temprepeat=~/^(.*?)\<inner\>(.*?)\<\/inner\>(.*?)$/si)) {
	$out.=$preinner;

	foreach my $photo (@thesephotos) {
	  &$dophoto($photo,$inner);
	}

	$count-=scalar @thesephotos;

	$temprepeat=$postinner;
      }
      $out.=$temprepeat;
      $count+=scalar @thesephotos;
    }
  }
  $template=$post;
}

$out.=$template;

if( $debug == 0 ) {
  # We now know everything for the headers
  &printheader();
}

print $out;

